<?php

namespace Jet_FB_MailPoet;

use Jet_FB_MailPoet\JetFormBuilder\Actions\Manager as JFBManager;
use Jet_FB_MailPoet\JetEngine\Notifications\Manager as JEManager;

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die();
}

class Plugin {
	/**
	 * Instance.
	 *
	 * Holds the plugin instance.
	 *
	 * @since 1.0.0
	 * @access public
	 * @static
	 *
	 * @var Plugin
	 */
	public static $instance = null;

	public $slug = 'jet-form-builder-mailpoet-action';

	public function __construct() {
		if ( class_exists( \MailPoet\API\API::class ) ) {
			$this->init_plugin();
		} else {
			add_action( 'admin_notices', function () {
				$class   = 'notice notice-error';
				$message = __(
					'<b>Error:</b> <b>JetFormBuilder MailPoet Action</b> plugin requires 
					<b><a href="https://wordpress.org/plugins/mailpoet/" target="_blank">MailPoet 3</a></b> plugin to work properly!',
					'jet-form-builder-mailpoet-action'
				);
				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), wp_kses_post( $message ) );
			} );
		}
	}

	public function init_plugin() {
		JFBManager::register();
		JEManager::register();

		Handler::instance();

		$can_init_license = (
			is_admin()
			&& function_exists( 'jet_form_builder' )
			&& array_key_exists( 'addons_manager', get_object_vars( jet_form_builder() ) )
		);

		if ( $can_init_license ) {
			require $this->plugin_dir( 'includes/class-jfb-license-manager.php' );

			new \JFB_License_Manager();
		}
	}

	public function get_version() {
		return JET_FB_MAILPOET_ACTION_VERSION;
	}

	public function plugin_url( $path ) {
		return JET_FB_MAILPOET_ACTION_URL . $path;
	}

	public function plugin_dir( $path = '' ) {
		return JET_FB_MAILPOET_ACTION_PATH . $path;
	}

	public function get_template_path( $template ) {
		$path = JET_FB_MAILPOET_ACTION_PATH . 'templates' . DIRECTORY_SEPARATOR;

		return ( $path . $template . '.php' );
	}

	/**
	 * Instance.
	 *
	 * Ensures only one instance of the plugin class is loaded or can be loaded.
	 *
	 * @return Plugin An instance of the class.
	 * @since 1.0.0
	 * @access public
	 * @static
	 *
	 */
	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

}

Plugin::instance();